use comfy::{
	draw_circle, draw_rect_outline, draw_sprite, main_camera_mut, EngineContext, IVec2,
	Vec2, RED, WHITE,
};
use log::info;
use comfy::is_key_pressed;
use comfy::is_key_down;
use comfy::KeyCode;

use crate::game::Ghost;
use crate::game::ZLayer;

pub mod worldgen;

pub fn draw(state: &crate::State, _engine: &comfy::EngineContext<'_>) {
	for (coords, tile) in state.overworld.iter_tiles() {
		for (i, texture) in tile.textures().iter().rev().enumerate() {
			let i = i as i32;
			draw_sprite(
				*texture,
				coords.as_vec2(),
				WHITE,
				ZLayer::MapMax - i,
				Vec2::ONE,
			);
			draw_rect_outline(coords.as_vec2(), Vec2::ONE, 0.1, RED, 10);
		}
	}
	draw_circle(
		state.ghost.overworld_pos.as_vec2(),
		0.5,
		RED,
		ZLayer::Ghost.into(),
	);
}

pub fn update(state: &mut crate::State, _engine: &mut EngineContext<'_>) {
	let mut camera = main_camera_mut();
	camera.center = Vec2::ZERO;
	camera.zoom = 30.0;

	let mut ghost_pos = &mut state.ghost.overworld_pos;

	// move player
	if is_key_down(KeyCode::Up) {
		ghost_pos.y +=1;
	}
	if is_key_down(KeyCode::Down) {
		ghost_pos.y -=1;
	}
	if is_key_down(KeyCode::Left) {
		ghost_pos.x -=1;
	}
	if is_key_down(KeyCode::Right) {
		ghost_pos.x +=1;
	}

	// generate more chunks if needed
	{
		let half_viewport = (camera.world_viewport() * 0.5 + 3.0).as_ivec2();
		state
			.overworld
			.get_or_generate_tile(*ghost_pos + IVec2::new(half_viewport.x, half_viewport.y));
		state
			.overworld
			.get_or_generate_tile(*ghost_pos + IVec2::new(half_viewport.x, -half_viewport.y));
		state
			.overworld
			.get_or_generate_tile(*ghost_pos + IVec2::new(-half_viewport.x, half_viewport.y));
		state
			.overworld
			.get_or_generate_tile(*ghost_pos + IVec2::new(-half_viewport.x, -half_viewport.y));
	}
}
